/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiBuilder_h__
#define __nuiBuilder_h__

#include "ngl.h"
#include "nglString.h"
#include <map>
#include <list>
#include "nuiContainer.h"

using namespace std;

class nuiXMLNode;
class nuiBuilder;

typedef class nuiBuilder* nuiBuilderPtr;

typedef nuiWidgetPtr (*nuiCreateWidgetFn)(nuiContainer *,nuiXMLNode *);
typedef std::map<nglString, nuiCreateWidgetFn> nuiBuilderMap;

class nuiBuilder
{
public:
  nuiBuilder();
  virtual ~nuiBuilder();

  virtual void Init();

  void SetHandler(const nglString& ClassName, nuiCreateWidgetFn pHandler); ///< This method permits to add or override a widget creation function.
  nuiCreateWidgetFn GetHandler(const nglString& ClassName); ///< This method retrieves thewidget creation function associated with a class name.
  bool GetClassList(std::list<nglString>& rClassNames); ///< This method fills the given nglString list with the name of the classes that this map can handle. 

protected:
   nuiBuilderMap mBuilderMap;
};

typedef nuiBuilder* nuiBuilderPtr;


nuiWidgetPtr nuiCreateWidget(nuiContainerPtr pParent, nuiXMLNode* pNode, nuiBuilderPtr pBuilder = NULL); ///< This method permits to create a real tree of widgets from an xml tree. You can optionally provider a pointer to a modified nuiBuilder that map class names to object creation functions, otherwise the default mapping will be automaticaly used.

#endif // __nuiBuilder_h__
