/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiButton_h__
#define __nuiButton_h__

#include "ngl.h"
#include "nuiContainer.h"
#include "nuiEvent.h"
#include "nuiImage.h"


/// This class implements a simple push button that acts as a container for a label, an image, or any other inactive widget element.
class nuiButton : public nuiContainer
{ 
public:
  nuiButton(nuiContainerPtr pParent); ///< Create a simple button (empty).
  nuiButton(nuiContainerPtr pParent, const nglString& rText); ///< Create a simple button with a nuiLabel as a child filling the client area.
  nuiButton(nuiContainerPtr pParent, const nglImage& rImage); ///< Create a simple button with a nuiImage as a child filling the client area.
  nuiButton(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create a button from an XML description.
  virtual ~nuiButton();

  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);

  // Rendering:
  virtual bool Draw(nuiDrawContext* pContext);

  // Public events:
  nuiSimpleEventSource < nuiButtonPressed > ButtonPressed;

  // Received Mouse events:
  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved  (nuiSize X, nuiSize Y);

  // Other atributes:
  nuiSize GetBorderSize(); ///< Retreive the current border around the area of the enclosed widget.
  void SetBorderSize(nuiSize BorderSize); ///< Set the current border around the area of the enclosed widget.
  void SetDrawSelf(bool val); ///< Permits to enable and disable the drawing of the button background and frame.
  bool GetDrawSelf(); ///< See SetDrawSelf

  // Class attributes:
  static nuiSize GetDefaultBorderSize(); ///< Retreive the default border around the area of the enclosed widget of every button that will be created.
  static void SetDefaultBorderSize(nuiSize BorderSize); ///< Set the default border around the area of the enclosed widget of every button that will be created.

protected:
  nuiBool mClicked;
  nuiSize mBorderSize;
  bool mDrawSelf;

  static nuiSize mDefaultBorderSize;
};

#endif // __nuiButton_h__
