/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiColor_h__
#define __nuiColor_h__

#include "ngl.h"
#include "nglString.h"
#include "nuiProperty.h"

/// This class implements a basic RGB color management class.
class nuiColor : public nuiValue
{
public:
  nuiColor(float r=0.0f,float g=0.0f, float b=0.0f, float a=1.0f)
  {
    Set(r,g,b,a);
    mValueType = nuiTypeColor;
  }

  nuiColor(const nuiColor& rColor)
  {
    mRed   = rColor.mRed;
    mGreen = rColor.mGreen;
    mBlue  = rColor.mBlue;
    mAlpha = rColor.mAlpha;
    mValueType = nuiTypeColor;
  }

  nuiColor(const nglString& rString)
  {
    SetValue(rString);
    mValueType = nuiTypeColor;
  }

  virtual ~nuiColor()
  {
  }

  inline float& Red()   { return mRed;   }
  inline float& Green() { return mGreen; }
  inline float& Blue()  { return mBlue;  }
  inline float& Alpha() { return mAlpha; }

  inline float GetRed() const   { return mRed;   }
  inline float GetGreen() const { return mGreen; }
  inline float GetBlue() const  { return mBlue;  }
  inline float GetAlpha() const { return mAlpha; }

  inline void Set(float r=0.0f,float g=0.0f, float b=0.0f, float a=1.0f)
  {
    mRed   = r;
    mGreen = g;
    mBlue  = b;
    mAlpha = a;
  }

  bool SetValue(const nglString& rString);
  void Get(nglString& rString) const;
  nglString GetValue() const;
  
  inline void Apply() const
  {
    glColor4f(mRed,mGreen,mBlue,mAlpha);
  }

protected:
  float mRed, 
        mGreen, 
        mBlue,
        mAlpha;
};

#endif // __nuiColor_h__
