/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiContainer_h__
#define __nuiContainer_h__

#include "nuiWidget.h"

class nuiContainer;

class nuiContainer : public nuiWidget
{
public:
  class Iterator
  {
  public:
    Iterator();
    Iterator(const Iterator& rIterator);
    nuiContainer::Iterator& operator = (const Iterator& rIterator);
    virtual ~Iterator();

    virtual bool IsValid();
    virtual nuiWidgetPtr GetWidget() const;
    virtual nuiWidgetPtr operator*() const
    {
      return GetWidget();
    }
  protected:
    nuiWidgetList::iterator mIterator;
    bool mValid;
  private:
    friend class nuiContainer;
    bool SetElement(const nuiWidgetList::iterator& rIterator);
    void SetValid(bool Valid);
    nuiWidgetList::iterator& GetElement();
  };

  typedef Iterator* IteratorPtr;

  /** @name Life */
  //@{
  nuiContainer(nuiContainerPtr pParent); ///< Create an nuiObject and give him his parent.
  nuiContainer(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create an nuiObject from an XML description.
  virtual ~nuiContainer(); 
  //@}

  /** @name Serialization */
  //@{
  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive = false);
  //@}

  /** @name Object relation management */
  //@{
  virtual bool AddChild(nuiWidgetPtr pChild); ///< Add a child to this object. This child will be added after the last child. Returns true if success.
  virtual bool DelChild(nuiWidgetPtr pChild,bool Delete=false); ///< Remove this child from the object. If Delete is true then the child will be deleted too. Returns true if success.
  virtual nuiWidgetPtr GetChild(int index); ///< Returns the child which has the given index (first child = 0). Return NULL in case of faillure.
  virtual nuiWidgetPtr GetChild(nuiSize X, nuiSize Y); ///< Returns the child which is under the pixel (X,Y) in this object or this if there is no such child. X and Y are given in the coordinate system of the parent object.
  virtual int GetChildrenCount(); ///< Returns the number of children this object has.
  virtual nuiWidgetPtr GetChild(const nglString& rName, bool recurse = true); ///< Find a child by its name property. Recurse the tree is asked politely (default = true).
  virtual bool Clear(bool Delete = true); ///< Clear all children. By default the children are deleted unless Delete == false.
  nuiContainerPtr GetRoot();
  nuiWidgetPtr Find (const nglString& rName); ///< Finds a node given its full path relative to the current node. Eg. Find("background/color/red").

  virtual nuiContainer::IteratorPtr GetFirstChild(); 
  virtual nuiContainer::IteratorPtr GetLastChild(); 
  virtual bool GetNextChild(nuiContainer::IteratorPtr pIterator); 
  virtual bool GetPreviousChild(nuiContainer::IteratorPtr pIterator); 
  //@}

  /** @name Outgoing events */
  //@{
  nuiSimpleEventSource<nuiChildAdded> ChildAdded;
  nuiSimpleEventSource<nuiChildDeleted> ChildDeleted;

  //@}

  // from nuiWidget:
  virtual void Invalidate(bool LayoutChanged = false, bool TellChildren = false);
  virtual bool Draw(nuiDrawContext* pContext);
  virtual nuiDrawContext* GetDrawContext();
  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);
  virtual void SetHover(bool Hover);

  /** @name Private event management system (do not override unless you know what you're doing!!!) */
  //@{
  /// Beware: these three methods receive the mouse coordinates in the repair of the root object!
  virtual bool DispatchMouseClick (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool DispatchMouseUnclick (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool DispatchMouseMove (nuiSize X, nuiSize Y);
  //@}

protected:

  bool DrawChildren(nuiDrawContext* pContext,bool ClipChildren = true); ///< This helper method calls the DrawMethod on all the children of the nuiObject.
  nuiWidgetList mpChildren;
};

#endif // __nuiContainer_h__
