/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/*!
\file  nuiFileSelector.h
\brief File selector
*/

#ifndef __nuiFileSelector_h__
#define __nuiFileSelector_h__

#include "nuiComposite.h"
#include "nuiEditLine.h"
#include "nuiFileTable.h"
#include "nuiWindow.h"


//! Simple file selector
/*
This composite widget does the usual combination of a file browser (based on nuiFileTable),
an entry (a nuiEditLine) and optionnally a popup window (a nuiWindow). It generates a single
event (\e Done) upon selection. Here is an example of the \e Done event handling :

\code
bool MyApp::OnFileSelectionDone(nuiEvent* pEvent)
{
  nglPath path;

  if (mpFileSelector->GetSelection(path))
  {
    // The user selected 'path'
  }
  return true;
}
\endcode
*/
class nuiFileSelector : public nuiComposite
{
public:
  nuiFileSelector(nuiContainerPtr pParent, const nglPath& rPath, bool PopUp = true);
  /*!< Creates a file selector instance
    \param pParent container
    \param rPath initial path displayed by the file browser
    \param PopUp if true, the widget will be wrapped into a nuiWindow widget (see SetTitle)
  */
  nuiFileSelector(nuiContainerPtr pParent, nuiXMLNode* pNode);
  virtual ~nuiFileSelector();

  void SetTitle(const nglString& rTitle);       ///< If built as a popup, set the popup's title
  void SetFileName(const nglString& FileName);  ///< Set the entry widget's current text

  bool GetSelection(nglPath& rPath);            ///< Retrieve selection. Only valid from the \e Done event handler

  nuiSimpleEventSource<nuiWidgetActivated> Done;  ///< Event triggered when the user was done (file selected or dialog closed/cancelled)

private:
  nuiWindow*     mpWin;
  nuiEditLine*   mpEntry;
  nuiFileTable*  mpTable;
  nglPath        mSelection;

  nuiEventSink<nuiFileSelector> mEventSink;

  bool OnClosed   (nuiEvent* pEvent);
  bool OnOK       (nuiEvent* pEvent);
  bool OnSelected (nuiEvent* pEvent);
};

#endif // __nuiFileSelector_h__
