/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiFileTable_h__
#define __nuiFileTable_h__

#include "nuiTable.h"

class nuiFileTable : public nuiTable
{
public:
  nuiFileTable(nuiContainerPtr pParent, const nglPath& rPath);
  nuiFileTable(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiFileTable();

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);

  void SetShowSize(bool Show);
  bool GetShowSize();
  void SetShowType(bool Show);
  bool GetShowType();
  void SetShowDirs(bool Show);
  bool GetShowDirs();
  void SetShowFiles(bool Show);
  bool GetShowFiles();

  bool Selected(nuiEvent* pEvent); ///< Called whenever a file is double clicked.
  bool KeyDown (nglKeyCode Key, nglChar Char); ///< Complementing nuiTable (Backspace, etc...).

  bool Populate(const nglPath& rPath);
  nglPath GetPath();
protected:
  nuiEventSink<nuiFileTable> mFileTableSink;

  nglPath mPath;

  bool mShowFiles;
  bool mShowDirs;

  bool mShowSize;
  bool mShowType;

  int mNameColumn;
  int mSizeColumn;
  int mTypeColumn;
};

#endif // __nuiFileTable_h__
