/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiFlags_h__
#define __nuiFlags_h__

#include "ngl.h"
#include "nglString.h"
class nuiXMLNode;

// Generic definitions that can be very useful for widgets:
enum nuiOrientation
{
  nuiHorizontal,
  nuiVertical
};

enum nuiDirection
{
  nuiForward,
  nuiBackward
};

enum nuiPosition
{
  nuiLeft,
  nuiRight,
  nuiTop,
  nuiBottom,
  nuiTopLeft,
  nuiTopRight,
  nuiBottomLeft,
  nuiBottomRight,
  nuiCenter,
  nuiTile,  ///< This position is not handled by most widgets. 
  nuiFill,
  nuiFillHorizontal,
  nuiFillVertical
};

enum nuiState
{
  nuiNormal,
  nuiSelected,
  nuiDisabled
};

// This is just plain ugly; i know...
typedef uint nuiWindowFlags;



/// Useful conversion functions:
nuiPosition     nuiGetPosition    (const nglString Position, nuiPosition Default = nuiCenter);
nuiDirection    nuiGetDirection   (const nglString Direction, nuiDirection Default = nuiForward);
nuiOrientation  nuiGetOrientation (const nglString Orientation, nuiOrientation Default = nuiVertical);
nuiState        nuiGetState       (const nglString State, nuiState Default = nuiNormal);

nglString nuiGetPosition (nuiPosition Value);
nglString nuiGetDirection (nuiDirection Value);
nglString nuiGetOrientation (nuiOrientation Value);
nglString nuiGetState (nuiState Value);

bool nuiGetBool (const nglString& rBool, bool Default = false);

// Get enum from XML desc:
nuiPosition     nuiGetPosition    (nuiXMLNode* pNode, const nglString Attr, nuiPosition Default = nuiCenter);
nuiDirection    nuiGetDirection   (nuiXMLNode* pNode, const nglString Attr, nuiDirection Default = nuiForward);
nuiOrientation  nuiGetOrientation (nuiXMLNode* pNode, const nglString Attr, nuiOrientation Default = nuiVertical);
nuiState        nuiGetState       (nuiXMLNode* pNode, const nglString Attr, nuiState Default = nuiNormal);

// Get enum from xml desc attrib with standard node:
nuiPosition     nuiGetPosition    (nuiXMLNode* pNode, nuiPosition Default = nuiCenter);
nuiDirection    nuiGetDirection   (nuiXMLNode* pNode, nuiDirection Default = nuiForward);
nuiOrientation  nuiGetOrientation (nuiXMLNode* pNode, nuiOrientation Default = nuiVertical);
nuiState        nuiGetState       (nuiXMLNode* pNode, nuiState Default = nuiNormal);

bool nuiGetBool (nuiXMLNode* pNode, const nglString Attr, bool Default = false);
nglString nuiGetString (nuiXMLNode* pNode, const nglString Attr, const nglString Default = nglString(""));

int     nuiGetVal (nuiXMLNode* pNode, const nglString Attr, int     Default = 0);
uint    nuiGetVal (nuiXMLNode* pNode, const nglString Attr, uint    Default = 0);
float   nuiGetVal (nuiXMLNode* pNode, const nglString Attr, float   Default = 0);
double  nuiGetVal (nuiXMLNode* pNode, const nglString Attr, double  Default = 0);

#endif // __nuiFlags_h__
