/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiLabel_h__
#define __nuiLabel_h__

#include "ngl.h"
#include "nglString.h"
#include "nuiWidget.h"
#include "nuiTheme.h"

/// This widget implements a text label. It can be used as a basic building block for buttons, list headers, list elements, etc...
class nuiLabel : public nuiWidget
{
public:
  nuiLabel(nuiContainerPtr pParent, const nglString& Text, nuiTheme::FontStyle FontStyle=nuiTheme::Default);
  nuiLabel(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiLabel();

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);

  void InitProperties();

  virtual bool Draw(nuiDrawContext* pContext);
  virtual nuiRect CalcIdealSize();

  virtual bool SetText(const nglString& Text); ///< Modify the label's text.
  virtual nglString GetText(); ///< Retreive the label's text.

  void SetTextColor(const nuiColor& Color, nuiState State = nuiNormal); ///< Get the text color for the given widget state.
  void SetBackgroundColor(const nuiColor& Color, nuiState State = nuiNormal); ///< Set the background color to use to clear the rect before drawing the text.
  nuiColor GetTextColor(nuiState State = nuiNormal); ///< Get the text color for the given widget state.
  nuiColor GetBackgroundColor(nuiState State = nuiNormal); ///< Get the background color to use to clear the rect before drawing the text. 
  void SetFontStyle(nuiTheme::FontStyle FontStyle); //< Change the font used by the widget.
  nuiTheme::FontStyle GetFontStyle(); //< Return the font used by the widget.

  void SetPosition(nuiPosition Position); ///< Set the position of the text in the widget's rectangle.
  nuiPosition GetPosition(); ///< Get the position of the text in the widget's rectangle.

  void SetBackground(bool bg); ///< If \param bg is true the label will clear its background before being drawn.
  bool GetBackground(); ///< Return true if the label will clear its background before being drawn.

protected:
  bool OnTextChanged(nuiEvent *pEvent);
  nuiEventSink<nuiLabel> mLabelSink; ///< event sink for property event

  nuiString mText;
  std::vector<nglString> mTextTokens;
  bool mTextChanged;
  nuiColor mColorNormal;
  nuiColor mColorNormalBg;
  nuiColor mColorSelected;
  nuiColor mColorDisabled;
  nuiColor mColorSelectedBg;
  nuiColor mColorDisabledBg;
  nuiBool mClearBg;
  nuiVPosition mPosition;
  nuiTheme::FontStyle mFontStyle;
};

#endif // __nuiLabel_h__
