/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiMenu_h__
#define __nuiMenu_h__

#include "nuiWindow.h"
#include "nuiEvent.h"

class nuiMenuItem;
class nuiScrollBar;

/// This class implements a simple popup menu. Remember that popup menu can only contain nuiMenuItems (this will change when I find a better design and have some time :). 
class nuiMenu : public nuiWindow
{
public:
  nuiMenu(nuiMainWindow* pParent, const nuiRect& rRect); ///< Create a menu in the given MainWindow. The rectangle's size is ignored as the menu will query is items to decide its size. 
  nuiMenu(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiMenu();

  // Rendering:
  virtual bool Draw(nuiDrawContext* pContext);
  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);

  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved  (nuiSize X, nuiSize Y);
  
  virtual bool AddItem(nuiMenuItem* pItem); ///< Add an item to the menu
  virtual bool IsParentActive();

  // Events:
  nuiSimpleEventSource<nuiMenuDone> MenuDone; ///< This event is launched when a menu entry have been selected and the menu is no longer needed OnScreen if SetTrashRemoval(false) have been called on the object.
  void SetTrashRemoval(bool); ///< If this is true the menu will be trashed as soon as a menu entry have been selected. Otherwise the MenuRemove call back will be called.

protected:
  bool MenuItemSelected(nuiEvent* pEvent); ///< This method receive the nuiMenuItemSelected event from the menu items. It must then handle the menu removal from the screen (see SetTrashRemoval() method).

  virtual void RemoveMenu(); ///< Handles the removal of the menu once an item have been selected.
private:
  virtual bool AddChild(nuiWidgetPtr pChild); ///< Prevent the addition of objects that are not inherited from nuiMenuItem!

  bool mTrashRemoval; ///< If this is true the menu will be trashed as soon as a menu entry have been selected. Otherwise the MenuRemove call back will be called.

  bool mEntered; ///< Anti reentrency flag.
  bool mFirstDraw; ///< This flag is set until the menu is drawn once. It is here to grab the mouse at the right time.
  nuiScrollBar* mpVScrollBar; ///< ScrollBar, needed when the menu size exceed the parent window size.
  nuiEventSink<nuiMenu> mMenuEventSink;
};

#endif // __nuiMenu_h__

