/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiMenuItem_h__
#define __nuiMenuItem_h__

#include "nuiContainer.h"
#include "nuiEvent.h"
#include "nuiColor.h"

class nuiMenu;

/// This class implements a container that is the basic compound for a popup menu.
class nuiMenuItem : public nuiContainer
{
public:
  nuiMenuItem(nuiMenu* pParent);
  nuiMenuItem(nuiMenu* pParent, const nglString& rText); ///< Create a menu item with a label.
  nuiMenuItem(nuiMenu* pParent, const nglImage& rImage); ///< Create a menu item with an image.
  nuiMenuItem(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  
  virtual ~nuiMenuItem();

  virtual bool Draw(nuiDrawContext* pContext); ///< Draw the menu item's background. 
  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);

  // Mouse Events:
  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved  (nuiSize X, nuiSize Y);

  nuiSimpleEventSource <nuiMenuItemSelected> ItemSelected; ///< This event is called whenever this menuitem is selected


};

/// This class implements a container that is the basic compound for a popup menu.
class nuiMenuSeparator : public nuiMenuItem
{
public:
  nuiMenuSeparator(nuiMenu* pParent);
  nuiMenuSeparator(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  
  virtual ~nuiMenuSeparator();

  bool Draw(nuiDrawContext* pContext); ///< Draw the menu item's background. 
  virtual nuiRect CalcIdealSize();
};

#endif // __nuiMenuItem_h__

