/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiMouseEvent_h__
#define __nuiMouseEvent_h__

#include "nuiEvent.h"
#include "nglWindow.h"
#include "nuiFlags.h"
#include "nuiProperty.h"
#include "nuiRect.h"

//////////// EVENTS:
/// This is a helper class to easily dispatch mouse clicked events
class nuiMouseClickedEvent : public nuiEvent
{
public:
  nuiMouseClickedEvent(nuiEventSource* pSource, nuiSize X, nuiSize Y, nglMouseFlags Button)
    : nuiEvent(pSource,nuiClicked)
  {
    mX = X;
    mY = Y;
    mButton = Button;
  }

  nuiSize mX;
  nuiSize mY;
  nglMouseFlags mButton;
};

/// This is a helper class to easily dispatch mouse unclicked events
class nuiMouseUnclickedEvent : public nuiEvent
{
public:
  nuiMouseUnclickedEvent(nuiEventSource* pSource, nuiSize X, nuiSize Y, nglMouseFlags Button)
    : nuiEvent(pSource,nuiUnclicked)
  {
    mX = X;
    mY = Y;
    mButton = Button;
  }

  nuiSize mX;
  nuiSize mY;
  nglMouseFlags mButton;
};

/// This is a helper class to easily dispatch mouse moved events
class nuiMouseMovedEvent : public nuiEvent
{
public:
  nuiMouseMovedEvent(nuiEventSource* pSource, nuiSize X, nuiSize Y)
    : nuiEvent(pSource,nuiMoved)
  {
    mX = X;
    mY = Y;
  }

  nuiSize mX;
  nuiSize mY;
};

//////// EVENT SOURCES:
/// This is a helper class to easily dispatch mouse clicked events
class nuiMouseClicked : public nuiEventSource
{
public:
  nuiMouseClicked()
  {
  }

  virtual bool operator() (nuiSize X, nuiSize Y, nglMouseFlags Button)
  {
    return SendEvent(new nuiMouseClickedEvent(this, X, Y, Button));
  }
};

/// This is a helper class to easily dispatch mouse unclicked events
class nuiMouseUnclicked : public nuiEventSource
{
public:
  nuiMouseUnclicked()
  {
  }

  virtual bool operator() (nuiSize X, nuiSize Y, nglMouseFlags Button)
  {
    return SendEvent(new nuiMouseUnclickedEvent(this, X, Y, Button));
  }
};

/// This is a helper class to easily dispatch mouse moved events
class nuiMouseMoved : public nuiEventSource
{
public:
  nuiMouseMoved()
  {
  }

  virtual bool operator() (nuiSize X, nuiSize Y)
  {
    return SendEvent(new nuiMouseMovedEvent(this, X, Y));
  }
};


#endif // __nuiMouseEvent_h__
