/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiRange_h__
#define __nuiRange_h__

#include "ngl.h"
#include "nuiEvent.h"
#include "nuiFlags.h"
#include "nuiProperty.h"
#include "nuiRect.h"


/// This class implements the behaviour of a range such as what is needed by a scroll bar for exemple. 
class nuiRange
{
public:
  nuiRange(nuiSize Value=0, nuiSize Min=0, nuiSize Max=50, nuiSize Increment=1, nuiSize PageIncrement=10, nuiSize PageSize=10);
  nuiRange(const nuiRange& rRange);
  virtual ~nuiRange();

  void SetValue(nuiSize Value);
  void SetRange(nuiSize Minimum,nuiSize Maximum);
  void SetIncrement(nuiSize Increment);
  void SetPageIncrement(nuiSize PageIncrement);
  void SetPageSize(nuiSize Size);

  void Increment();
  void Decrement();
  void PageIncrement();
  void PageDecrement();
  bool MakeInRange(nuiSize number);

  nuiSize GetValue();
  nuiSize GetMinimum();
  nuiSize GetMaximum();
  nuiSize GetIncrement();
  nuiSize GetPageIncrement();
  nuiSize GetPageSize();
  nuiSize GetRange();

  nuiSimpleEventSource<nuiValueChanged> ValueChanged; ///< This event is sent whenever the value is changed.
  nuiSimpleEventSource<nuiChanged> Changed; ///< This event is sent whenever the range is changed (not the value).

protected:
  nuiSize mMinimum;
  nuiSize mMaximum;
  nuiSize mPageSize;
  nuiSize mValue;
  nuiSize mIncrement;
  nuiSize mPageIncrement;

};

#endif // __nuiRange_h__
