/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiSVGView_h__
#define __nuiSVGView_h__

#include "nui.h"
#include "ngl.h"
#include "nuiXML.h"
#include "nuiDrawContext.h"
#include "nuiSVGShape.h"

class nuiSVGView : public nuiWidget
{
public:
  nuiSVGView(nuiContainerPtr pParent, nuiSVGShape* pShape = NULL); ///< Load a svg view from an existing svg shape.
  nuiSVGView(nuiContainerPtr pParent, const nglPath& rShapePath); ///< Construct a svg view from a SVG xml description file.
  nuiSVGView(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Load from an XML description.
  virtual ~nuiSVGView();

  bool Load (nglIStream& rSource);
	
  virtual bool Draw(nuiDrawContext* pContext);

  virtual nuiRect CalcIdealSize();

  void EnableAntialias(bool Antialias);
  bool IsAntialiasEnabled();

private:
  nuiSVGShape* mpShape;
  bool mAntialias;

  void Init();
};

#endif // __nuiSVGView_h__
