/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiScrollBar_h__
#define __nuiScrollBar_h__

#include "nuiWidget.h"
#include "nuiEvent.h"
#include "nuiTimer.h"
#include "nuiRange.h"

#define SBSIZE 12 // Basic standard size... Should go in the theme!

/// Basic scroll bar with range support. Used in too many compound widgets to name :).
class nuiScrollBar : public nuiWidget
{
public:
  nuiScrollBar(nuiContainerPtr pParent, nuiOrientation orientation, const nuiRange& rRange = nuiRange());
  nuiScrollBar(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiScrollBar();

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);

  virtual bool Draw(nuiDrawContext* pContext);

  virtual nuiRect CalcIdealSize();

  virtual nuiRange& GetRange(); ///< Return the Range used by this scroll bar. 
  virtual void SetOrientation(nuiOrientation Orientation); ///< Change the orientation of the scroll bar
  virtual nuiOrientation GetOrientation(); ///< Return the orientation of the scroll bar

  //Received Events:
  bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  bool MouseMoved  (nuiSize X, nuiSize Y);

  // Sent events:
  nuiSimpleEventSource<nuiValueChanged> ValueChanged; ///< This event is sent whenever the scrollbar's thumb position is changed (by the user or by the program).
  bool HandlePageDown(nuiEvent* pEvent);
  bool HandlePageUp(nuiEvent* pEvent);
protected:
  bool DoInvalidate(nuiEvent* pEvent);

  nuiVOrientation mOrientation;

  nuiSize mClickValue;

  bool mClicked;
  bool mThumbClicked;
  bool mPageUpClicked;
  bool mPageDownClicked;

  nuiSize mClickX;
  nuiSize mClickY;

  nuiRange mRange;

  nuiTimer mTimer;
  nuiEventSink<nuiScrollBar> mScrollBarSink;
};

#endif // __nuiScrollBar_h__
