/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiSplitter_h__
#define __nuiSplitter_h__

#include "ngl.h"
#include "nuiContainer.h"
#include "nuiEvent.h"
#include "nuiTimer.h"

/// This is a split view widget. It divides its client area in two views (horizontaly or verticaly) which acts as two subcontainers. The user can change the proportional size of the area, but it is also possible to lock the split to prevent modification.
class nuiSplitter : public nuiContainer
{
public:
  nuiSplitter ( nuiContainerPtr pParent, nuiOrientation orientation);
  nuiSplitter ( nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.

  virtual ~nuiSplitter();

  void InitProperties();

  virtual bool Draw(nuiDrawContext* pContext);

  virtual nuiRect CalcIdealSize();
  virtual bool SetRect(const nuiRect& rRect);

  void SetOrientation(nuiOrientation orientation);
  nuiOrientation SetOrientation();

  nuiSize GetHandlePos(); ///< Retreive the Position of the handle separating the 2 widgets (in %).
  void SetHandlePos(nuiSize HandlePos); ///< Set the the size of the handle separation the 2 widgets (int %).

  nuiSize GetHandlePosMin(); ///< Retreive the minimum Position of the handle separating the 2 widgets (in %).
  void SetHandlePosMin(nuiSize HandlePos); ///< Set the the minimum position of the handle separation the 2 widgets (int %).
  nuiSize GetHandlePosMax(); ///< Retreive the maximum Position of the handle separating the 2 widgets (in %).
  void SetHandlePosMax(nuiSize HandlePos); ///< Set the the maximum position of the handle separation the 2 widgets (int %).

  void GotoHandlePos(nuiSize HandlePos, nuiSize handleStep = 1, double timerStep = 0.02); ///< go to handle pos

  nuiSize GetHandleSize(); ///< Retreive the size of the handle separation the 2 widgets.
  void SetHandleSize(nuiSize HandleSize); ///< Set the the size of the handle separation the 2 widgets.

  void SetFixed(bool flag); ///< Enable or disable mouse modifications of the splitter widget.
  bool GetFixed(); ///< Returns true if mouse modifications of the splitter widget is enabled.
  void SetMasterChild(bool MasterChild); ///< We will try to display the first widget completely if true.
  bool GetMasterChild(); ///< Return true if MasterChild is enabled.

  // Other atributes:
  nuiSize GetBorderSize(); ///< Retreive the current border around the area of the enclosed widget.
  void SetBorderSize(nuiSize BorderSize); ///< Set the current border around the area of the enclosed widget.

  // Received Mouse events:
  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved  (nuiSize X, nuiSize Y);

  // Class attributes:
  static nuiSize GetDefaultBorderSize(); ///< Retreive the default border around the area of the enclosed widget of every button that will be created.
  static void SetDefaultBorderSize(nuiSize BorderSize); ///< Set the default border around the area of the enclosed widget of every button that will be created.

protected:
  bool StepHandlePos(nuiEvent *pEvent);  
  nuiVOrientation mOrientation; /// mHorizontal is true if the containers' layout if horizontal, false if vertical.
  std::list<nuiRect> mPositions;
  nuiFloat mBorderSize;
  nuiFloat mHandleSize;
  nuiBool mClicked;
  nuiBool mFixed;
  nuiBool mHandleHover;

  nuiFloat mHandlePos;
  nuiFloat mHandlePosMin;
  nuiFloat mHandlePosMax;

  nuiTimer *mpTimer; ///< timer for GotoHandlePos()
  nuiFloat mStartHandlePos; ///< original position
  nuiFloat mEndHandlePos; ///< end position
  nuiFloat mGotoStep; ///< step
  nuiEventSink<nuiSplitter> mSplitterSink; ///< event sink for timer

  nuiBool mMasterChild;

  static nuiSize mDefaultBorderSize;
protected:
};

#endif // __nuiSplitter_h__
