/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiTexture_h__
#define __nuiTexture_h__

#include "ngl.h"
#include "nglImage.h"
#include "nuiWidget.h"
#include "nuiRect.h"
#include "nuiColor.h"
#include "nuiRefCount.h"

#include <map>

/// This class implements a basic image widget.
class nuiTexture : public nuiObject, public nuiRefCount
{
public:
  // Constructors and destructors are protected!
  static nuiTexture* GetTexture (nglIStream* pInput, nglImageCodec* pCodec = NULL); ///< Create an image from an input stream and a codec.  If \param pCodec is NULL all codecs will be tried on the image.
  static nuiTexture* GetTexture (const nglPath& rPath, nglImageCodec* pCodec = NULL ); ///< Create an image from a path and a codec. If \param pCodec is NULL all codecs will be tried on the image.
  static nuiTexture* GetTexture (nglImageInfo& rInfo, bool Clone = true); ///< Create an image from an nglImageInfo structure. If \param is true then the image buffer will be cloned, otherwise it will be deleted with the nuiTexture.
  static nuiTexture* GetTexture (const nglImage& rImage); ///< Create an image by copying an existing nglImage.
  static nuiTexture* GetTexture (nglImage* pImage, bool OwnImage); ///< Create an image from an existing nglImage. If \param OwnImage the nglImage object will be deleted with the nuiTexture.
  static nuiTexture* GetTexture (nuiXMLNode* pNode); ///< Create an image from an xml description.

  void Release();

  virtual nuiXMLNode* Serialize(nuiXMLNode* pParentNode, bool Recursive);

  virtual void ForceReload(); ///< This method deletes the texture assiciated with the nuiTexture thus forcing its recreation at the next rendertime.
  virtual void ImageToTextureCoord(float& x, float& y); ///< Transform the x,y point in the coordinates of the image to the coordinates of the texture. 
  virtual void TextureToImageCoord(float& x, float& y); ///< Transform the x,y point in the coordinates of the texture to the coordinates of the image. 

  virtual nglImage* GetImage(); ///< Return a pointer to the nglImage contained in this object.
protected:
  nuiTexture (nglIStream* pInput, nglImageCodec* pCodec = NULL); ///< Create an image from an input stream and a codec.  If \param pCodec is NULL all codecs will be tried on the image.
  nuiTexture (const nglPath& rPath, nglImageCodec* pCodec = NULL ); ///< Create an image from a path and a codec. If \param pCodec is NULL all codecs will be tried on the image.
  nuiTexture (nglImageInfo& rInfo, bool Clone = true); ///< Create an image from an nglImageInfo structure. If \param is true then the image buffer will be cloned, otherwise it will be deleted with the nuiTexture.
  nuiTexture (const nglImage& rImage); ///< Create an image by copying an existing nglImage.
  nuiTexture (nglImage* pImage, bool OwnImage); ///< Create an image from an existing nglImage. If \param OwnImage the nglImage object will be deleted with the nuiTexture.
  nuiTexture (nuiXMLNode* pNode); ///< Create an image from an xml description.
  
  virtual ~nuiTexture();
  friend class nuiDrawContext;
  virtual bool Apply(nuiDrawContext* pContext); ///< Set this texture as the current opengl texture. 

  nglImage* mpImage;
  bool mOwnImage;
  GLuint mTexture;
  nuiSize mRealWidth;
  nuiSize mRealHeight;
  bool mForceReload;

  static std::map<nglString,nuiTexture*> mpTextures;
};

#endif // __nuiTexture_h__
