/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiTheme_h__
#define __nuiTheme_h__

#include "ngl.h"
#include "nglPath.h"
#include "nuiObject.h"
#include "nuiContainer.h"

class nuiDrawContext;
class nuiWindow;
class nuiButton;
class nuiScrollBar;

class nglFont1;

enum nuiWidgetElement
{
  eActiveWindowShade,
  eInactiveWindowShade,

  eActiveWindowBg,
  eInactiveWindowBg,

  eActiveWindowFg,
  eInactiveWindowFg,

  eScrollBarFg,
  eScrollBarBg,

  eNormalButtonBg,
  eSelectedButtonBg,
  eDisabledButtonBg,
  eHoverNormalButtonBg,
  eHoverSelectedButtonBg,
  eHoverDisabledButtonBg,

  eTableColumnBg,
  eSelectedTableColumnBg,
  eSelectedTableLineBg,

  eLastWidgetElement
};

/// This class implements the nui theme system.
class nuiTheme: public nuiObject
{
public:
  nuiTheme(const nglPath& rPath);
  nuiTheme(nuiXMLNode* pNode);
  virtual ~nuiTheme();

  bool Load(nuiXMLNode* pNode);

  enum FontStyle
  {
    Default =0,
    Fixed =1,
    StyleCount = 2
  };

  // Window decorations:
  virtual void DrawWindowShade (nuiDrawContext* pContext, const nuiRect& rRect, bool Active);
  virtual void DrawWindow(nuiDrawContext* pContext, nuiWindow* pWindow); ///< Render a normal window border
  virtual void DrawActiveWindow(nuiDrawContext* pContext, nuiWindow* pWindow); ///< Render a Top Level window border
  virtual void DrawMovingWindow(nuiDrawContext* pContext, nuiWindow* pWindow);
  virtual void AdjustWindowRect(nuiRect& rRect, nuiWindowFlags Flags);
  virtual bool IsInsideWindow(nuiRect Rect,nuiSize X,nuiSize Y,nuiWindowFlags Flags);
  virtual nuiPosition GetWindowPart(nuiRect rect,nuiSize X,nuiSize Y,nuiWindowFlags Flags);
  virtual void DrawWindowBackground(nuiDrawContext* pContext, nuiWindow* pWindow, nuiColor& DefaultColor);

  // Scrollbar:
  virtual void DrawScrollBarBackground(nuiDrawContext* pContext, nuiScrollBar* pScroll);
  virtual void DrawScrollBarForeground(nuiDrawContext* pContext, nuiScrollBar* pScroll);

  // Buttons
  virtual void DrawButton(nuiDrawContext* pContext, nuiButton* pButton);  ///< Render a button border

  // Menus a Menu items:
  virtual void DrawMenuWindow(nuiDrawContext* pContext, const nuiRect& rRect); ///< draw the a menu window (border, shadow, etc...).
  virtual void DrawMenuItem(nuiDrawContext* pContext, const nuiRect& rRect, bool Hover); ///< Draw the background of a menu item.

  // Element Colors:
  const nuiColor& GetElementColor(nuiWidgetElement Element) const;
  void SetElementColor(nuiWidgetElement Element,const nuiColor& rColor);
protected:
  friend class nuiDrawContext;
  // Fonts and text handling:
  void SetFont(FontStyle Style, nglFont1* pFont, bool Delete=true); ///< Changes the font used for the given style. If Delete==true the old one will be deleted.
  nglFont1* GetFont(FontStyle Style); ///< Retreive the font used for the given style.

  virtual void SetTextColor(const nuiColor& rColor, FontStyle Style = Default);
  virtual void DrawText(nuiSize x, nuiSize y, const nglString& rString, FontStyle Style = Default); /// Draw text at the given coordinates and the given font style.

  void LoadDefaults(); ///< Load the default color & geometry values.

  nglFont1* mpFonts[StyleCount];

  nuiColor mTableColumnBG;
  nuiColor mSelectedTableColumnBG;
  nuiColor mSelectedTableLineBG;

  nuiColor mButtonBorder[2][2];
  nuiColor mButtonFill[2][2];

  nuiColor mElementColors[eLastWidgetElement];
};

#endif // __nuiTheme_h__
