/*
  NUI - C++ cross-platform GUI framework for OpenGL based applications
  Copyright (C) 2002-2003 Sbastien Mtrot

  This library is free software; you can redistribute it and/or
  modify it under the terms of the GNU Lesser General Public
  License as published by the Free Software Foundation; either
  version 2.1 of the License, or (at your option) any later version.

  This library is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public
  License along with this library; if not, write to the Free Software
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef __nuiUserArea_h__
#define __nuiUserArea_h__

#include "ngl.h"
#include "nuiContainer.h"
#include "nuiEvent.h"
#include "nuiMouseEvent.h"

/// This widget implements a user area. It draws basicaly nothing by itself and fires a UserDraw event instead. It up to the programmer to catch this event and fill the area with whatever opengl code he wants. 
class nuiUserArea : public nuiWidget
{
public:
  nuiUserArea(nuiContainerPtr pParent);
  nuiUserArea(nuiContainerPtr pParent, nuiXMLNode* pNode); ///< Create from an XML description.
  virtual ~nuiUserArea();

  virtual bool Draw(nuiDrawContext* pContext);

  // Out going Events
  nuiSimpleEventSource<nuiUserDraw> UserDraw;
  nuiMouseClicked   ClickedMouse;
  nuiMouseUnclicked UnclickedMouse;
  nuiMouseMoved     MovedMouse;

  virtual bool MouseClicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseUnclicked  (nuiSize X, nuiSize Y, nglMouseFlags Button);
  virtual bool MouseMoved  (nuiSize X, nuiSize Y);

  virtual void SetInterceptMouse(bool intercept=false); ///< If intercept==true then all the mouse event will be intercepted by the UserArea. By default no event is intercepted.
  virtual bool GetInterceptMouse(); ///< Return the mouse event intercption state.


protected:
  nuiBool mInterceptMouse;
};

#endif // __nuiUserArea_h__
